unit main;

{$mode objfpc}{$H+}

interface

uses
Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, DateUtils, INIFiles, LazUTF8Classes;

type

{ TfrmMain }

TfrmMain = class(TForm)
mLog: TMemo;
procedure FormCreate(Sender: TObject);
procedure FormDestroy(Sender: TObject);
procedure LoadVariables(INIFile: TIniFile; Section: String);
procedure MakePlaylist();
private

public

end;

var
frmMain: TfrmMain;
search_last: Boolean;
work_dir, save_dir, playlist_name, playlist_header, name_mask, file_ext, config_file,
stop_name, stop_date_name, start_name: String;
before_text, after_text, work_sections, old_log: TStringList;
playlist: TStringListUTF8;
log_file: File;
ini_file: TIniFile;
days_ago, name_offset, i: Integer;

implementation

{$R *.lfm}

{ TfrmMain }

//Процедура загрузки переменных
procedure TfrmMain.LoadVariables(INIFile: TIniFile; Section: String);
var
    str_i: Integer;
begin
    search_last := INIFile.ReadBool(Section, 'search_last', True);                          //Искать ли старый файл
    work_dir := INIFile.ReadString(Section, 'work_dir', ExtractFilePath(ParamStr(0)));      //Папка в которой ищутся файлы
    save_dir := INIFile.ReadString(Section, 'save_dir', work_dir);                          //Папка сохраненния плейлиста
    playlist_name := INIFile.ReadString(Section, 'playlist_name', 'playlist.m3u8');          //Название плейлиста
    playlist_header := INIFile.ReadString(Section, 'playlist_header', '#EXTM3U');           //Заголовок плейлиста
    name_mask := INIFile.ReadString(Section, 'name_mask', 'yy-mm-dd');                      //Формат имени файла для поиска
    name_offset := INIFile.ReadInteger(Section, 'name_offset', 0);                         //Смещение числа в имени на сколько дней назад
    file_ext := INIFile.ReadString(Section, 'file_ext', 'mp3');                             //Расширение файла
    days_ago := INIFile.ReadInteger(Section, 'days_ago', 7);                                //За сколько дней назад искать файл, если нужный не найден
    stop_name := INIFile.ReadString(Section, 'stop_name', 'STOP');                          //Имя стоп файла
    stop_date_name := INIFile.ReadString(Section, 'stop_date_name', 'STOP-');               //Имя стоп файла с конкретной датой
    start_name := INIFile.ReadString(Section, 'start_name', 'START-');                      //Имя старт файла с конкретной датой
//Создаем стринглист для файлов которые будут вписаны в плейлист ДО искомого файла
    str_i := 0;
//Перебираем переменные что бы все необходимые добавить в плейлист
    while INIFile.ValueExists(Section, 'before_text_'+IntToStr(str_i)) do begin
          before_text.Add(INIFile.ReadString(Section, 'before_text_'+IntToStr(str_i), ''));
          str_i := str_i+1;
    end;
//Создаем стринглист для файлов которые будут вписаны в плейлист ПОСЛЕ искомого файла
    after_text := TStringList.Create;
    str_i := 0;
//Перебираем переменные что бы все необходимые добавить в плейлист
    while INIFile.ValueExists(Section, 'after_text_'+IntToStr(str_i)) do begin
          after_text.Add(INIFile.ReadString(Section, 'after_text_'+IntToStr(str_i), ''));
          str_i := str_i+1;
    end;
    mLog.Lines.Add(FormatDateTime('c', Now)+': Config for '+Section+' load!');
//Создадим папки если их нет, ато мало ли
    ForceDirectories(save_dir);
    ForceDirectories(work_dir);
end;

//Процедура поиска файла и создание плейлиста
procedure TfrmMain.MakePlaylist();
var
    current_date, need_file: String;
    DecDay, c: Integer;
begin
// Получаем текущую дату
   current_date := FormatDateTime(name_mask, Today+name_offset);
// Формируем необходимое имя файла
   need_file := current_date+'.'+file_ext;

   mLog.Lines.Add(FormatDateTime('c', Now)+': Need file '+need_file);

// Создаем плейлист
   playlist := TStringListUTF8.Create;
// Добавляем в него шапку
   playlist.AddStrings(playlist_header);

//Если есть стоп файл, то не ищем последний файл
   if FileExists(work_dir+stop_name) then begin
       mLog.Lines.Add(stop_name+' found in '+work_dir);
       search_last := False;
   end;
//Если есть стоп файл с датой и она сегодня, то не ищем последний файл
   if FileExists(work_dir+stop_date_name+current_date) then begin
       mLog.Lines.Add(stop_date_name+current_date+' found in '+work_dir);
       search_last := False;
   end;
//Если есть старт файл с датой и она сегодня, то ищем последний файл
   if FileExists(work_dir+start_name+current_date) then begin
       mLog.Lines.Add(start_name+current_date+' found in '+work_dir);
       search_last := True;
   end;
//Ищем необходимый файл
   if not FileExists(work_dir+need_file) then begin
       mLog.Lines.Add(FormatDateTime('c', Now)+': File '+need_file+' not found!');
//Если не нашли, то ищем последний файл по дате
       if search_last then begin
//Сбрасываем название необходимого файла, ибо будем формировать его заново
           need_file := '';
//Начинаем искать файлы с предыдущей даты, начиная от вчера и заканчивая сегодня-days_ago
           for DecDay := 1 to days_ago do begin
               mLog.Lines.Add(FormatDateTime('c', Now)+': Try to find old file '+work_dir+FormatDateTime(name_mask, Today+name_offset-DecDay)+'.'+file_ext+'...');
//Проверка на существования старого файла
               if FileExists(work_dir+FormatDateTime(name_mask, Today+name_offset-DecDay)+'.'+file_ext) then begin
//Если он существует, то записываем его имя и выходим из поиска
                  need_file := work_dir+FormatDateTime(name_mask, Today+name_offset-DecDay)+'.'+file_ext;
                  Break;
               end;
           end;
//Проверка, если нашли старый файл
           if need_file <> '' then begin
               mLog.Lines.Add(FormatDateTime('c', Now)+': Last file found! Add '+need_file+' to playlist.');
//Добавляем в плейлист файлы до, нужный файл и после файлы и сохраняем
               playlist.AddStrings(before_text);
               playlist.Add(need_file);
               playlist.AddStrings(after_text);
               for c := playlist.Count - 1 downto 0 do
               begin
                 if Trim(playlist[c]) = '' then
                   playlist.Delete(c);
               end;
               try
                  playlist.SaveToFile(save_dir+playlist_name);
               except
                     // Обработка ошибки
                     on E:Exception do
                                  mLog.Lines.Add(FormatDateTime('c', Now)+': Error save playlist: '+E.Message);
               end;

           end else begin
//Если и старый файл не нашли, сохраняем пустой плейлист
               mLog.Lines.Add(FormatDateTime('c', Now)+': Last file NOT found! Save empty playlist.');
               try
                  playlist.SaveToFile(save_dir+playlist_name);
               except
// Обработка ошибки
                     on E:Exception do
                      mLog.Lines.Add(FormatDateTime('c', Now)+': Error save playlist: '+E.Message);
               end;
           end;
       end else begin
//Если поиск старого файла отключен, сохраняем пустой плейлист
           mLog.Lines.Add(FormatDateTime('c', Now)+': Save empty playlist...');
           try
              playlist.SaveToFile(save_dir+playlist_name);
           except
               // Обработка ошибки
                 on E:Exception do
                            mLog.Lines.Add(FormatDateTime('c', Now)+': Error save playlist: '+E.Message);
           end;
       end;
   end else begin
//Актуальный файл нашли, добавляем его и файлы до\после в плейлист и сохраняем.
       mLog.Lines.Add(FormatDateTime('c', Now)+': Need file found and add to playlist.');
       playlist.AddStrings(before_text);
       playlist.AddStrings(work_dir+need_file);
       playlist.AddStrings(after_text);
       for c := playlist.Count - 1 downto 0 do
       begin
         if Trim(playlist[c]) = '' then
           playlist.Delete(c);
       end;
       try
          playlist.SaveToFile(save_dir+playlist_name);
       except
// Обработка ошибки
             on E:Exception do
             mLog.Lines.Add(FormatDateTime('c', Now)+': Error save playlist: '+E.Message);
       end;
   end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin

   old_log := TStringList.Create;

   config_file:=ExtractFilePath(ParamStr(0))+'config.ini';
   mLog.Lines.Add(FormatDateTime('c', Now)+': PlayList Maker Started' );
   mLog.Lines.Add(FormatDateTime('c', Now)+': Looking for config in '+config_file);

 // Проверяем есть ли конфиг файл
   while not FileExists(config_file) do begin
       mLog.Lines.Add(FormatDateTime('c', Now)+': Config not found and create now.');
       ini_file := TINIFile.Create(config_file);
       LoadVariables(ini_file, 'Default');
       ini_file.WriteBool('Default', 'search_last', search_last);
       ini_file.WriteString('Default', 'work_dir', work_dir);
       ini_file.WriteString('Default', 'save_dir', save_dir);
       ini_file.WriteString('Default', 'playlist_name', playlist_name);
       ini_file.WriteString('Default', 'playlist_header', playlist_header);
       ini_file.WriteString('Default', 'name_mask', name_mask);
       ini_file.WriteInteger('Default', 'name_offset', name_offset);
       ini_file.WriteString('Default', 'file_ext', file_ext);
       ini_file.WriteInteger('Default', 'days_ago', 7);
       ini_file.WriteString('Default', 'stop_name', stop_name);
       ini_file.WriteString('Default', 'stop_date_name', stop_date_name);
       ini_file.WriteString('Default', 'start_name', start_name);
       ini_file.WriteString('Default', 'before_text_0', '');
       ini_file.WriteString('Default', 'after_text_0', '');
   end;
   mLog.Lines.Add(FormatDateTime('c', Now)+': Config found and go to load.');
   ini_file := TINIFile.Create(config_file);
   if ( ParamStr(1) = '' ) or (not ini_file.SectionExists(ParamStr(1))) then begin
       if FileExists('PLMaker.log') then
         old_log.LoadFromFile('PLMaker.log');
         old_log.Insert(0, '');

       mLog.Lines.Add(FormatDateTime('c', Now)+': Program run with clear or not exists section, load all sections for work...');
       work_sections := TStringList.Create;
       ini_file.ReadSections(work_sections);
       if work_sections.Count > 0 then begin
           mLog.Lines.Add(FormatDateTime('c', Now)+': Found '+IntToStr(work_sections.Count)+' sections. Go to work!');
           for i:= 0 to work_sections.Count-1 do begin
               before_text := TStringList.Create;
               after_text := TStringList.Create;
               LoadVariables(ini_file, work_sections.Strings[i]);
//work
               MakePlaylist();
               before_text.Free;
               after_text.Free;
           end;
       end else begin
           mLog.Lines.Add(FormatDateTime('c', Now)+': Found 0 sections. Stop working.');
       end;
   end else begin
     if FileExists(ParamStr(1)+'.log') then
         old_log.LoadFromFile(ParamStr(1)+'.log');
     old_log.Insert(0, '');
       mLog.Lines.Add(FormatDateTime('c', Now)+': Load '+ParamStr(1)+' section for work...');
                      before_text := TStringList.Create;
               after_text := TStringList.Create;
       LoadVariables(ini_file, ParamStr(1));
//work
       MakePlaylist();
                      before_text.Free;
               after_text.Free;
   end;
   Application.Terminate;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
   mLog.Lines.Add('-----------------------------------------------------------------------');
   mLog.Lines.Add('-----------------------------STOP LOG----------------------------------');
   mLog.Lines.Add('-----------------------------------------------------------------------');
   mLog.Lines.AddStrings(old_log);
   if ( ParamStr(1) = '' ) or (not ini_file.SectionExists(ParamStr(1))) then begin
        try
           mLog.Lines.SaveToFile('PLMaker.log');
        except
              on E:Exception do begin
              end;
        end;
   end else begin
        try
           mLog.Lines.SaveToFile(ParamStr(1)+'.log');
        except
              on E:Exception do begin
              end;
        end;
   end;
end;

end.

